
import pandas as pd
from ase.io import read
from glob import glob

def get_calculated_value(infile: str, term: str, valueindex: int) -> any:
	valueList = []
	with open(infile) as gfile:
		line = gfile.readline()
		while line != "":
			if term in line:
				pieces = line.split()
				valueList.append(pieces[valueindex])
			line = gfile.readline()
	return valueList[-1]

def get_total_dipole_moment(infile: str, term: str, valueindex: int) -> float: #from inputsmd.log 
	with open(infile) as gfile:
		line = gfile.readline()
		status = False
		while line != "" and not status:
			if term in line:
				line = gfile.readline()
				pieces = line.split()
				searchedValue = float(pieces[valueindex])
				status = True
			line = gfile.readline()
	return searchedValue

def get_smiles(infile: str) -> str:
	with open(infile) as f:
		line = f.readline()
	return line.split()[0]

def get_occupied_energy_alpha_levels(infile: str):
	alphaOccupiedLevelList = []
	with open(infile) as gfile:
		line = gfile.readline()
		while line != "":
			if "Alpha  occ. eigenvalues" in line:
				pieces = line.split()
				for value in pieces[4:]:
					alphaOccupiedLevelList.append(float(value))
			line = gfile.readline()
	return alphaOccupiedLevelList

def get_occupied_energy_beta_levels(infile: str):
	betaOccupiedLevelList = []
	with open(infile) as gfile:
		line = gfile.readline()
		while line != "":
			if "Beta  occ. eigenvalues" in line:
				pieces = line.split()
				for value in pieces[4:]:
					betaOccupiedLevelList.append(float(value))
			line = gfile.readline()
	return betaOccupiedLevelList 

def get_unoccupied_energy_alpha_levels(infile: str):
	alphaUnoccupiedLevelList = []
	with open(infile) as gfile:
		line = gfile.readline()
		while line != "":
			if "Alpha virt. eigenvalues" in line:
				pieces = line.split()
				for value in pieces[4:]:
					alphaUnoccupiedLevelList.append(float(value))
			line = gfile.readline()
	return alphaUnoccupiedLevelList

def get_unoccupied_energy_beta_levels(infile: str):
	betaUnoccupiedLevelList = []
	with open(infile) as gfile:
		line = gfile.readline()
		while line != "":
			if "Beta virt. eigenvalues" in line:
				pieces = line.split()
				for value in pieces[4:]:
					betaUnoccupiedLevelList.append(float(value))
			line = gfile.readline()
	return betaUnoccupiedLevelList 


term1 = "NAtoms=  " #1
term2 = "Stoichiometry" #-1
term3 = "Dipole moment (field-independent basis, Debye):" #-1
term4 = "GePol: Cavity volume"  #-2
term5 = "Thermal correction to Gibbs Free Energy=" #-1
term6 = "SCF Done:" #4



from glob import glob
dirs = glob("../01_Data/*")

outFile = open('dataset_pt.csv', 'w')
outFile.write('moleculeID;electState;numOfAtoms;chemicalFormula;chemicalWeight;volume;ksOccupiedUp;ksOccupiedUp-1;ksOccupiedDown;ksOccupiedDown-1;ksUnoccupiedUp;ksUnoccupiedUp+1;ksUnoccupiedDown;ksUnoccupiedDown+1;SMILES;pKa\n')

for dirname in dirs:
	print(f'Reading {dirname} ...')
	for evalue, pvalue, state, pka in [(0, 1, 'AH2', 'pKa03 ='),(+1, 2, 'AH21+', 'pKa14 ='),(+2, 1, 'AH22+', 'pKa25 ='),(-1, 1, 'AH1-', 'pKa36 ='),(0, 2, 'AH', 'pKa47 ='),(+1, 1, 'AH1+', 'pKa58 =')]:
		sqsch = f"{dirname}/schemesquare.dat"
		pm7file = f"{dirname}/{state}/input.log"
		m06file = f"{dirname}/{state}/inputsmd.log"
		smifile = f"{dirname}/{state}/pos.smi"

		pKa = get_calculated_value(sqsch, pka, -1)
		moleculeID = dirname.split("/")[-1]
		numOfAtoms = int(get_calculated_value(pm7file, term1, 1))
		chemicalFormula = get_calculated_value(pm7file, term2, -1)

		moleculeInfo = read(pm7file, format="gaussian-out")
		moleculeAtomCoordinates = moleculeInfo.get_positions()
		masses = moleculeInfo.get_masses()
		moleculeWeight = sum(masses)
		dipoleMoment = get_total_dipole_moment(m06file, term3 , -1)
		moleculeCavityVolums = float(get_calculated_value(m06file, term4, -2))

		gibbsCorrection = float(get_calculated_value(pm7file, term5, -1))
		totalEnergy = float(get_calculated_value(m06file, term6, 4))
		
		smiles = get_smiles(smifile)
		onlyAlphaStates = ['A', 'AH2', 'A2-', 'AH1+', 'AH1-', 'AH22+']

		if not state in onlyAlphaStates:
			alphaOccupiedLevelList = get_occupied_energy_alpha_levels(m06file)
			betaOccupiedLevelList = get_occupied_energy_beta_levels(m06file)
			ksOccUp = alphaOccupiedLevelList[-1]
			ksOccUpMinusOne = alphaOccupiedLevelList[-2]
			ksOccDown = betaOccupiedLevelList[-1]
			ksOccDownMinusOne = betaOccupiedLevelList[-2]

			alphaUnoccupiedLevelList = get_unoccupied_energy_alpha_levels(m06file)
			betaUnoccupiedLevelList = get_unoccupied_energy_beta_levels(m06file)
			ksUnoccUp = alphaUnoccupiedLevelList[0]
			ksUnoccUpPlusOne = alphaUnoccupiedLevelList[1]
			ksUnoccDown = betaUnoccupiedLevelList[0]
			ksUnoccDownPlusOne = betaUnoccupiedLevelList[1]

		else:
			alphaOccupiedLevelList = get_occupied_energy_alpha_levels(m06file)
			ksOccUp = alphaOccupiedLevelList[-1]
			ksOccUpMinusOne = alphaOccupiedLevelList[-2]
			ksOccDown = ksOccUp
			ksOccDownMinusOne = ksOccUpMinusOne

			alphaUnoccupiedLevelList = get_unoccupied_energy_alpha_levels(m06file)
			ksUnoccUp = alphaUnoccupiedLevelList[0]
			ksUnoccUpPlusOne = alphaUnoccupiedLevelList[1]
			ksUnoccDown = ksUnoccUp
			ksUnoccDownPlusOne = ksUnoccUpPlusOne

		outFile.write(f"{moleculeID};{evalue};{numOfAtoms};{chemicalFormula};{moleculeWeight};{moleculeCavityVolums};{ksOccUp};{ksOccUpMinusOne};{ksOccDown};{ksOccDownMinusOne};{ksUnoccUp};{ksUnoccUpPlusOne};{ksUnoccDown};{ksUnoccDownPlusOne};{smiles};{pKa}\n")

outFile.close()
